"
COPYRIGHT (c) 2020 LabWare

stx:libscm - a new source code management library for Smalltalk/X

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License. 

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
"
"{ Package: 'stx:libscm/common' }"

"{ NameSpace: Smalltalk }"

Object subclass:#SCMCopyrightLine
	instanceVariableNames:'line prefix years holder'
	classVariableNames:''
	poolDictionaries:''
	category:'SCM-Common-StX-Tools'
!

!SCMCopyrightLine class methodsFor:'documentation'!

copyright
"
COPYRIGHT (c) 2020 LabWare

stx:libscm - a new source code management library for Smalltalk/X

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License. 

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
"
! !

!SCMCopyrightLine class methodsFor:'instance creation'!

new
    "return an initialized instance"

    ^ self basicNew initialize.
!

readFrom:aStringOrStream onError:aBlock 
    "Parse a copyright line from given `aStringOrStream` (line). If given
     stream does not contain copyright line, return value of `aBlock`"
    
    | l  s  c  p  y1  y2 |

    l := aStringOrStream readStream contents.
    s := l readStream.
     
    "/ skip leading whitespaces:    
    
    s skipSeparators.
     
    "/ skip (optional) asterisk and whitespace following it.        
    s peek == $* ifTrue:[
        s next.
        s skipSeparators.
    ].
    "/ skip (optional) hash and whitespace following it.        
    s peek == $# ifTrue:[
        s next.
        s skipSeparators.
    ].
     
    "/ look for 'Copyright (c)' string. If found, this is a copyright
    "/ line, otherwise it is not and we return `nil` to indicate that.
    
    c := (s "'COPYRIGHT (c) ' size" nextAvailable:14) asLowercase.
    c = 'copyright (c) ' ifFalse:[
        ^ aBlock value
    ].
    p := s position.
     
    "/ parse years
    
    s skipSeparators.
    s peek isDigit ifTrue:[
        y1 := y2 := s nextAvailable:4.
        s skipSeparators.
        s peek == $- ifTrue:[
            s next.
            s skipSeparators.
            y2 := (s nextAvailable:4) trimSeparators.
            s skipSeparators.
        ].
        (y1 allSatisfy:[:c | c isDigit ]) ifTrue:[
            y1 := y1 asNumber.
        ].
        (y2 allSatisfy:[:c | c isDigit ]) ifTrue:[
            y2 := y2 asNumber.
        ].
    ].
    ^ (SCMCopyrightLine new)
        prefix:(l copyTo:p - 1);
        years:(y1 notNil ifTrue:[
                    y1 to:y2
                ] ifFalse:[ nil ]);
        holder:(l copyFrom:s position + 1);
        yourself

    "
     SCMCopyrightLine readFrom: ' COPYRIGHT (c) 1994 by Claus Gittinger'
     SCMCopyrightLine readFrom: '              All Rights Reserved'

     SCMCopyrightLine readFrom: ' * COPYRIGHT (c) 1988-1995 by Claus Gittinger'
     SCMCopyrightLine readFrom: ' *              All Rights Reserved'

     SCMCopyrightLine readFrom: ' * COPYRIGHT (c) 2011-now Jan Vrany'
     SCMCopyrightLine readFrom: ' *              All Rights Reserved'

     SCMCopyrightLine readFrom: ' # COPYRIGHT (c) 2011-now Jan Vrany'

     SCMCopyrightLine readFrom: ' COPYRIGHT (c) Claus Gittinger / eXept Software AG'"

    "Created: / 14-05-2018 / 16:12:52 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified (comment): / 27-05-2020 / 11:33:26 / Jan Vrany <jan.vrany@labware.com>"
! !

!SCMCopyrightLine class methodsFor:'utilities'!

collectCopyrightsFromText:lines 
    "
    Collect all copyrights from given lines and return
    a collection of `SCMCopyrightLine`s...
    "                 
    | current  copyrights  lineNr |

    copyrights := OrderedCollection new.
    current := nil.
    lineNr := 1.
    [ lineNr <= lines size ] whileTrue:[
        | line  copyright |

        line := lines at:lineNr.
        copyright := SCMCopyrightLine readFrom:line onError:[ nil ].
        copyright notNil ifTrue:[
            copyright line:lineNr.
            current isNil ifTrue:[
                current := OrderedCollection with:copyright.
            ] ifFalse:[
                current add:copyright
            ].
        ] ifFalse:[
            current notNil ifTrue:[
                copyrights add:current.
                current := nil.
            ].
        ].
        lineNr := lineNr + 1.
    ].
    current notNil ifTrue:[
        copyrights add:current.
        current := nil.
    ].
    ^ copyrights

    "Created: / 26-05-2020 / 10:11:51 / Jan Vrany <jan.vrany@labware.com>"
! !

!SCMCopyrightLine methodsFor:'accessing'!

holder
    ^ holder
!

holder:something
    holder := something.
!

line
    ^ line
!

line:something
    line := something.
!

prefix
    ^ prefix ? ' COPYRIGHT (C)'

    "Modified: / 15-05-2018 / 13:16:28 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

prefix:something
    prefix := something.
!

years
    ^ years
!

years:something
    years := something.
! !

!SCMCopyrightLine methodsFor:'comparing'!

< another
    years ~= another years ifTrue:[ 
        ^ years start < another years start
            or:[ years stop < another years stop ]
    ] ifFalse:[ 
        ^ holder < another holder
    ].

    "Created: / 14-05-2018 / 16:54:21 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

= another
    ^ self class == another class
        and: [ prefix = another prefix
        and: [ years = another years 
        and: [ holder = another holder ]]]

    "Created: / 14-05-2018 / 16:50:14 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

hash

    "/ This is a really bad hash function, we need #hashMultiply:
    "/ For now...
    ^ prefix hash bitXor: (years hash bitXor: holder hash)

    "Created: / 14-05-2018 / 16:48:50 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!SCMCopyrightLine methodsFor:'converting'!

asString
    ^ String streamContents:[ :s|
        s nextPutAll: self prefix; space.
        years notNil ifTrue:[
            years start printOn: s.
            years start ~~ years stop ifTrue:[
                s nextPut:$-.
                years stop printOn: s.        
            ].
            s space.
        ].
        s nextPutAll: holder.
    ].

    "Created: / 14-05-2018 / 15:45:33 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 15-05-2018 / 13:16:38 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!SCMCopyrightLine methodsFor:'initialization'!

initialize
    "Invoked when a new instance is created."
    | thisYear |

    thisYear := Date today year.

    prefix := 'COPYRIGHT (c)'.
    years := thisYear to: thisYear.
    holder := UserPreferences current historyManagerCopyrightHolder .
    "/ super initialize.   -- commented since inherited method does nothing

    "
    SCMCopyrightLine new
    "

    "Modified: / 27-05-2020 / 13:24:56 / Jan Vrany <jan.vrany@labware.com>"
! !

!SCMCopyrightLine methodsFor:'printing & storing'!

printOn:aStream
    "append a printed representation of the receiver to the argument, aStream"

    super printOn:aStream.
    aStream nextPut:$(.
    aStream nextPutAll: self asString.
    aStream nextPut:$).

    "Modified: / 14-05-2018 / 15:50:44 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!SCMCopyrightLine methodsFor:'queries'!

canBeMergedWith: anotherCopyright
    | myFrom myTo anotherFrom anotherTo |

    (self isYearToNow or:[ anotherCopyright isYearToNow ]) ifTrue:[ 
        ^ false.
    ].
    holder ~= anotherCopyright holder ifTrue:[ 
        ^ false
    ].
    myFrom := years first.
    myTo := years last.
    anotherFrom := anotherCopyright years first.
    anotherTo := anotherCopyright years last.

    ^ (anotherFrom <= (myTo + 1) and: [anotherTo >= (myFrom - 1)])
    "
    (SCMCopyrightLine new years: (2018 to: 2019)) canBeMergedWith: (SCMCopyrightLine new years: (2020 to: 2020))
    (SCMCopyrightLine new years: (2017 to: 2018)) canBeMergedWith: (SCMCopyrightLine new years: (2020 to: 2020))
    (SCMCopyrightLine new years: (2017 to: 2018)) canBeMergedWith: (SCMCopyrightLine new years: (2015 to: 2020))
    (SCMCopyrightLine new years: (2017 to: 2018)) canBeMergedWith: (SCMCopyrightLine new years: (2015 to: 2016))
    (SCMCopyrightLine new years: (2017 to: 2018)) canBeMergedWith: (SCMCopyrightLine new years: (2015 to: 2015))
    "

    "Created: / 27-05-2020 / 12:30:56 / Jan Vrany <jan.vrany@labware.com>"
! !

!SCMCopyrightLine methodsFor:'testing'!

isYearToNow
    "
    Return true, if this copyright years is in form YYYY-now (such as 20017-now).
    These are now considered invalid
    "
    ^ years notNil and:[ years stop = 'now' ]

    "Created: / 15-05-2018 / 06:14:13 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!SCMCopyrightLine methodsFor:'utilities'!

merge: anotherCopyright

    self assert: (self canBeMergedWith: anotherCopyright).  
    ^ SCMCopyrightLine new
        line: line ? anotherCopyright line;
        prefix: prefix;
        holder: holder;
        years: ((years first min: anotherCopyright years first) 
                    to: (years last max: anotherCopyright years last));
        yourself

    "
    (SCMCopyrightLine new years: (2018 to: 2019)) merge: (SCMCopyrightLine new years: (2020 to: 2020))
    (SCMCopyrightLine new years: (2017 to: 2018)) merge: (SCMCopyrightLine new years: (2015 to: 2020))
    (SCMCopyrightLine new years: (2017 to: 2018)) merge: (SCMCopyrightLine new years: (2015 to: 2016))
    "

    "Created: / 27-05-2020 / 12:45:44 / Jan Vrany <jan.vrany@labware.com>"
! !

